<?php declare (strict_types = 1);

namespace EuKit\Base\ViewModels;

use EuKit\Base\Parts\NewsMLArticle;
use EuKit\Base\ViewModels\BasePage;
use Everyware\Concepts\Concepts;
use Infomaker\Everyware\Base\Models\Page;
use Infomaker\Everyware\Base\OpenContent\OpenContentProvider;
use Infomaker\Everyware\Base\OpenContent\QueryBuilder;
use Infomaker\Everyware\Support\Date;
use Infomaker\Everyware\Twig\View;
use OcAPI;
use OpenContent;
use Tightenco\Collect\Support\Arr;
use EuKit\Base\Paginator;
use EuKit\Base\ViewModels\Teaser;

class SearchPageAdv extends BasePage
{
    private const SORT_NEWEST = 'newest';
    private const SORT_OLDEST = 'oldest';
    private const OC_SORT_FIELD = 'Pubdate';
    private $limit = 10;
    private $queryArray;
    private $queryData;
    // Variable Names
    private $OCVariables = [];
    private $facetData = [];
    //Dynamic Query params
    private $dateRange;
    private $pagenr = 1;
    private $sort;
    private $q; // search keyword
    private $allcheck = [];
    private $allfields = [];

    private $oc_url; // OC base url
    private $start = 0;
    private $facet_limit = 10;
    private $and_appender = '&';

    private $provider;
    private $startdate;
    private $enddate;
    private $config_data = [];

    public function __construct()
    {
        $page = new Page();
        parent::__construct($page);
        
		//Returns queryArray
		if (parse_url($_SERVER['REQUEST_URI'], PHP_URL_QUERY)) {
            parse_str(parse_url($_SERVER['REQUEST_URI'], PHP_URL_QUERY), $this->queryArray);
        }

        //Page Limit
        $this->setLimit(Arr::get($_GET, 'limit'));
        //Page Start
        $this->start = $this->pagenr * $this->limit - $this->limit;
        //Current Page
        $this->pagenr = (int) Arr::get($_GET, 'pagenr', 1);
        //Sort Order
        $this->sort = Arr::get($_GET, 'sort');

        //Search Keyword
        $this->q = Arr::get($_GET, 'q');
        $this->q = stripslashes(str_replace('\\', '', preg_replace('/\//', '', $this->q)));
		$server_time_zone = date_default_timezone_get();
        date_default_timezone_set($server_time_zone);

        //Date Range
        $this->dateRange = Arr::get($_GET, 'daterange');

        //Base OC Url
        $ocApi = new OcAPI();
        $this->oc_url = $ocApi->getOcBaseUrl();
        $this->include_config_facets();
        $this->facet_to_twig();
        $this->getArticles();

        //Set Data to Twig
        $this->setViewData('limit', $this->limit);
        $this->setViewData('dateRange', $this->dateRange);
        $this->setViewData('pagenr', $this->pagenr);
        $this->setViewData('sort', $this->sort);
        $this->setViewData('advsortOrders', $this->getSortings());
        $this->setViewData('tealiumGroup', 'searchpage');
        $this->setViewData('facetData', $this->facetData);
        $this->setViewData('queryparams', $this->queryArray);

    }

	/**
	 * Function to convert JSON into Array, values are in the array "facetData".
	 */
    public function include_config_facets()
    {
        // We will use this code in the config-facets.json file
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => get_template_directory_uri() . '/config-facets.json',
            CURLOPT_SSL_VERIFYPEER => false, //-k
            CURLOPT_SSL_VERIFYHOST => false,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => false,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'GET',
        ));

        $response = curl_exec($curl);
        curl_close($curl);
        
        // Ref: https://naviga-hub.atlassian.net/wiki/spaces/CP/pages/9711026217/Advance+Search+Feature

        //$response = '';

        if (!empty($response)) {
            $config_data = json_decode($response, true);
            $facetItems = $config_data['facet']['facetItems'];
            $conjson = Concepts::init();
            foreach ($facetItems as $field_key => $field_value) {
                $this->OCVariables[] = $field_key;
                //if ($field_value['fieldName'] == "wbmtypes-name") {
                if ( $field_key == "ConceptCategoryUuids") {
                    $field_values = $fc = [];
                    
                    $field_values = ( $field_value['IncludedFieldsStatus'] == "TRUE" )?$field_value['IncludedFields']:$this->get_included_fields($field_key); 
                    $fuuid = [];
                    foreach ($field_values as $fv) {
                        $fuuid[] = $fv; 
                        $uuidarr = $conjson->findByUuid($fv);
                        $fc[] = isset($uuidarr->post_title) ? $uuidarr->post_title : '';
                    }
                    $this->facetData[] = [
                        'OCvar' => $field_key,
                        'Name' => $field_value['Name'],
                        'fieldName' => $field_value['fieldName'],
                        'uuid' => !empty($fuuid)?$fuuid:$field_value['IncludedFields'],
                        'IncludedFields' => $fc,
                    ];

                } else {
                    $this->facetData[] = [
                        'OCvar' => $field_key,
                        'Name' => $field_value['Name'],
                        'fieldName' => $field_value['fieldName'],
                        'IncludedFields' => ($field_value['IncludedFieldsStatus'] == "TRUE")?$field_value['IncludedFields']:$this->get_included_fields($field_key)
                    ];
                }
            }
        }
    }

	/**
	 * Facets to be included in the query
	 */
    public function include_facet()
    {
        $inclusion_query = $this->and_appender . "facet.indexfield=" . "Pubdate";
        foreach ($this->OCVariables as $ocvar) {
            $inclusion_query .= $this->and_appender . "facet.indexfield=" . $ocvar;
        }
        return $inclusion_query;
    }

	/**
	 * Return the array of filters for the passed index field. 
	 */
    public function get_included_fields($field_key) {
		$search_query = $this->oc_url
		. 'search?'
		.'facet=true'
		.'&facet.indexfield='
		.$field_key;
		$search_query .= $this->and_appender . 'Pubdate=' . $this->encoder($this->getDateFilter());
		//$search_query .= $this->exclusion_query(); //only for WBM
		$response_array = $this->execute_search_query( $search_query );
   
    	$return_array = [];
		if( !empty($response_array) ){
			foreach( $response_array['facet']['fields'] as $ra ){
				foreach( $ra['frequencies'] as $term_name ){
					$return_array[] = $term_name['term'];
				}
			} 
		}
    	return array_filter($return_array);
   }

	/**
	 * Exclude these documents
	 */
    public function exclusion_query()
    {
        return $this->and_appender . "fq=" . "-CustomerTemplateType:" . "Email"
        . $this->and_appender . "fq=" . "-CustomerTemplateType:" . "PDF"
        . $this->and_appender . "fq=" . "-CustomerTemplateType:" . "Advert"
        . $this->and_appender . "fq=" . "-CustomerTemplateType:" . $this->encoder('"Quote of the day"')
        . $this->and_appender . "fq=" . "-CustomerTemplateType:" . $this->encoder('"Chart of the day"')
        . $this->and_appender . "fq=" . "-CustomerTemplateType:" . $this->encoder('"Company Profile"')
        . $this->and_appender . "fq=" . "-CustomerTemplateType:" . $this->encoder('"Embed content"')
        . $this->and_appender . "fq=" . "-CustomerArticleOnHold:" . $this->encoder('"on hold"');
    }

	/**
	 * Encode Url params
	 */
    public function encoder($query)
    {
        if (!empty($query)) {
            return rawurlencode(htmlspecialchars_decode($query));
        }

    }

	/**
	 * Exeute Search query
	 */
    public function execute_search_query( $search_query )
    {
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $search_query,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'GET',
            CURLOPT_HTTPHEADER => array(
                "Authorization: Basic " . base64_encode(OpenContent::getInstance()->getOcUserName() . ":" . OpenContent::getInstance()->getOcPassword())),
        ));

        $response = curl_exec($curl);
        curl_close($curl);

        return json_decode($response, true) ?? [];
    }

	/**
	 * Passes the facetData values into Twig.
	 */
    public function facet_to_twig()
    {
        $this->queryData = [];
        $TwigType = [];
        $fields = [];
       
        $con = Concepts::init();
        foreach ($this->facetData as $ocb) {
            $this->allfields[] = $ocb['fieldName'];
            if (!empty($this->queryArray[$ocb['fieldName']])) {
                $fields[] = $ocb['fieldName'];
                $i = -1;
                if (!empty($this->queryArray[$ocb['fieldName']])) {
                    foreach ($this->queryArray[$ocb['fieldName']] as $fn) {
                        if (!empty($fn)) {
                            ++$i;
                            $uuidarr = $con->findByUuid($fn);
                            if (isset($uuidarr->post_title)) {
                                $this->queryData[$ocb['fieldName']][$i]['term'] = $uuidarr->post_title;
                                $this->queryData[$ocb['fieldName']][$i]['uuid'] = $fn;
                                continue;
                            }
                            $this->queryData[$ocb['fieldName']][$i] = $fn;
                        }
                    }
                }
            }
        }
        $this->setViewData('selwbtname', $this->queryData);
        $this->setViewData('fields', $fields);
    }

    private function setLimit(?string $limit): void
    {
        $limit = (int) $limit;

        if (!$limit || $limit < 0) {
            return;
        }

        $this->limit = $limit > 100 ? 100 : $limit;
    }

    private function getArticles()
    {
        $q = !empty($this->q) ? $this->q . ' AND ' : '';
        $this->provider = OpenContentProvider::setup([
            'q' => $q . QueryBuilder::where('contenttype', 'Article')
                ->buildQueryString(),
            'limit' => $this->limit,
            'start' => $this->pagenr * $this->limit - $this->limit,
            'facet' => 'true',
            'sort.indexfield' => self::OC_SORT_FIELD,
            'sort.Pubdate.ascending' => $this->get_sort_order(),
            'fq' => $this->getfacets(),
            'Pubdate' => $this->getDateFilter(),
        ]);
        $this->provider->setPropertyMap('Article');
        $articles = array_map(static function (NewsMLArticle $article) {
            return (new Teaser($article))->getViewData();
        }, $this->provider->queryWithRequirements());
        
        $this->setViewData('articles', $articles);
        $this->setViewData('paginator', Paginator::make($this->provider->hits(), $this->limit));

        $current_url = $_SERVER['REQUEST_URI'];

		$current_date = date('m/d/Y');
        $sevendays = date('m/d/Y', strtotime('-7 days')) . " - " . $current_date;
        $thirtydays = date('m/d/Y', strtotime('-30 days')) . " - " . $current_date;        
        $twentyhrs = date('m/d/Y', strtotime('-1 days')) . " - " . $current_date;
        $pastyear = date('m/d/Y', strtotime('-365 days')) . " - " . $current_date;

        $this->setViewData('current_url', $current_url);
        $this->setViewData('sevenday', $sevendays);
        $this->setViewData('thirtydays', $thirtydays);
        $this->setViewData('twentyhrs', $twentyhrs);
        $this->setViewData('pastyear', $pastyear);

        $this->setViewData('startdate', $this->startdate);
        $this->setViewData('enddate', $this->enddate);
        $this->setViewData('allcheck', $this->allcheck);
        
        if (empty($this->dateRange)) {
            $this->setViewData('customdate', 'nodates');
        } elseif ($this->dateRange != $pastyear && $this->dateRange != $twentyhrs && $this->dateRange != $thirtydays && $this->dateRange != $sevendays) {
            $this->setViewData('customdate', 'customdate');
            $this->setViewData('customcheck', 'checked');
        } else {
            $this->setViewData('customdate', 'nodates');
        }
    }

    private function getfacets() //: string 
    {
        if (!empty($this->queryArray)) {
            $i=0;
        	foreach( $this->allfields as $af  ) {
               	if(  empty($this->queryArray[$af] )  ) 
               	$this->allcheck[] = $af;
                if(  isset($this->queryArray[$af] ) && is_array($this->queryArray[$af] ) ) {
					foreach($this->queryArray[$af] as $aaf){
						if( empty($aaf) )
							$this->allcheck[] = $af;
					}
                }
                  
        	}
   	 	}
		else {
			$this->allcheck =  $this->allfields;
		}

		if (!empty($this->queryArray)) {
			$ar = [];
			foreach ($this->queryArray as $key => $value) {
				if (is_array($value) && !empty($value)) {
					foreach ($value as $v) {
						if (!empty($v)) {
							$ar[$key][] = $v;
						}
					} //foreach
				} //if
			}//foreach
		}
    
		$ch = '';
		if (!empty($ar)) {
			foreach ($this->facetData as $ocb) {
				if (!empty($ar[$ocb['fieldName']])) {
					$ch .= $ocb['OCvar'] . ':("';
					$ch .= implode('" OR "', $ar[$ocb['fieldName']]);
					$ch .= '") AND ';
				} else {
					$ch .= "";
				}
			}
		} else {
			$ch .= "";
		}
		$ch = rtrim($ch, " AND ");
		return htmlspecialchars_decode($ch);
	} // function ends

	private function getDateFilter(): string
	{
		$startDate = '*';
		$endDate = '*';
		if ($this->dateRange) {
			$dateRange = explode(' - ', $this->dateRange);
			$start = $dateRange[0].date('H:i:s') ?? null;
			$end = $dateRange[1].date('H:i:s') ?? null;
			$startDate = $start ? Date::parse($start)->toIso8601ZuluString() : $startDate;
			$endDate = $end ? Date::parse($end)->toIso8601ZuluString() : $endDate;
			setcookie('dtr', $_GET['daterange'], time() + 86400, "/");
		}
		return "[{$startDate} TO {$endDate}]";
	}

	private function get_sort_order()
	{
		if (empty($this->sort) || $this->sort == self::SORT_NEWEST) {
			return 'false';
		} else {
			return 'true';
		}

	}

	public function getSortings(): array
	{
		return [
			self::SORT_NEWEST => ucfirst(self::SORT_NEWEST),
			self::SORT_OLDEST => ucfirst(self::SORT_OLDEST),
		];
	}

	public function render()
	{
        $isAdvancedSearchPage = true;
		View::render('@base/page/advanced-search', array_replace($this->getViewData(), [
			'metaGroup' => 'advanced-search',
            'advance_search_page_custom_js' => $isAdvancedSearchPage,
		]));
	}
}
